from jury.metrics._core import AutoMetric, MetricForTask, list_metrics, load_metric
from jury.metrics.accuracy import Accuracy
from jury.metrics.bertscore import Bertscore
from jury.metrics.bleu import Bleu
from jury.metrics.f1 import F1
from jury.metrics.meteor import Meteor
from jury.metrics.precision import Precision
from jury.metrics.recall import Recall
from jury.metrics.rouge import Rouge
from jury.metrics.sacrebleu import Sacrebleu
from jury.metrics.squad import Squad

__all__ = [
    "Accuracy",
    "AutoMetric",
    "Bertscore",
    "Bleu",
    "F1",
    "Meteor",
    "Precision",
    "Recall",
    "Rouge",
    "Sacrebleu",
    "Squad",
    "load_metric",
    "list_metrics",
]



from typing import List, Union

import numpy as np

from jury.utils import NestedSingleType


class Collator(list):
    def __init__(self, sequence, keep=False):
        sequence = self._construct(sequence, keep=keep)
        super().__init__(sequence)

    @property
    def shape(self):
        return np.array(self, dtype=object).shape

    @property
    def ndim(self):
        return len(self.shape)

    def collapse(self):
        return Collator(np.ravel(self).tolist(), keep=True)

    def nested(self):
        return Collator(self.from_list(self))

    def reshape(self, *args):
        _seq = np.array(self, dtype=object)
        if _seq.shape[:2] == (1, 1):
            return Collator(_seq.ravel().reshape(1, -1).tolist(), keep=True)
        elif _seq.ndim == 3 and _seq.shape[1] == 1:
            args = tuple(list(args) + [-1])
        return Collator(_seq.reshape(args).tolist(), keep=True)

    def reshape_len(self, *args):
        _len = len(self)
        return self.reshape(_len, *args)

    def can_collapse(self):
        if self.ndim >= 2:
            return self.shape[1] == 1
        if isinstance(self[0], list):
            n_item = len(self[0])
            return all([len(items) == n_item for items in self])
        return True

    def to_list(self, collapse=True):
        if collapse:
            return list(self.collapse())
        return list(self)

    def _construct(self, sequence: Union[str, List[str], List[List[str]]], keep: bool) -> List[List[str]]:
        if keep:
            return sequence

        _type_primary = NestedSingleType.get_type(sequence, order=0)
        _type_secondary = NestedSingleType.get_type(sequence, order=1)
        if _type_primary in ["str", "dict"]:
            sequence = self.from_str(sequence)
        elif _type_primary == "list" and _type_secondary != "list":
            sequence = self.from_list(sequence)

        return sequence

    @staticmethod
    def from_list(seq: List[str]):
        return [[item] for item in seq]

    @classmethod
    def from_str(cls, seq: str):
        return [seq]



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Metrics base class. The part of this file is adapted from HuggingFace's
datasets package implementation of Accuracy metric. See
https://github.com/huggingface/datasets/blob/master/src/datasets/metric.py
"""

from abc import ABC, abstractmethod
from typing import Any, Callable, Dict, List, Optional, Tuple, Union

import datasets
import numpy
import numpy as np
import pandas as pd
from datasets.utils.logging import get_logger

from jury.collator import Collator
from jury.metrics._core.utils import import_module, is_reduce_fn

LanguageGenerationInstance = Union[List[str], List[List[str]]]
SequenceClassificationInstance = Union[List[int], List[List[int]]]
SequenceLabelingInstance = List[List[str]]
EvaluationInstance = Union[LanguageGenerationInstance, SequenceClassificationInstance, SequenceLabelingInstance]
MetricOutput = Dict[str, Union[str, int, float]]

logger = get_logger(__name__)


class Metric(datasets.Metric, ABC):
    """
    Base metric class and common API for all metrics.

    Args:
        task (``str``): Task for the metric to be used. Tasks differ in inputs of predictions or references.
        resulting_name (Optional ``[str]``): Optional resulting name for :py:class:`jury.Jury` to use. By default, it
            uses `metric.name` if not given. This is meant to prevent clashes for output dict of
            :py:method:`jury.Jury.evaluate` such as when bleu-1, and bleu-2 are used together.
        compute_kwargs (Optional ``Dict[str, Any]``): These are the parameters to be passed to compute function of the
            metric. It is meant to ease the support of computation from a jury configuration file, etc.
        config_name (Optional ``str``): This is used to define a hash specific to a metrics computation script and
            prevents the metric's data to be overridden when the metric loading script is modified.
        keep_in_memory (``bool``): keep all predictions and references in memory. Not possible in distributed settings.
        cache_dir (Optional ``str``): Path to a directory in which temporary prediction/references data will be stored.
            The data directory should be located on a shared file-system in distributed setups.
        num_process (``int``): specify the total number of nodes in a distributed settings.
            This is useful to compute metrics in distributed setups (in particular non-additive metrics like F1).
        process_id (``int``): specify the id of the current process in a distributed setup (between 0 and num_process-1)
            This is useful to compute metrics in distributed setups (in particular non-additive metrics like F1).
        seed (Optional ``int``): If specified, this will temporarily set numpy's random seed when
        :func:`datasets.Metric.compute` is run.
        experiment_id (Optional ``str``): A specific experiment id. This is used if several distributed evaluations
            share the same file system. This is useful to compute metrics in distributed setups (in particular
            non-additive metrics like F1).
        max_concurrent_cache_files (``int``): Max number of concurrent metrics cache files (default 10000).
        timeout (``Union[int, float]``): Timeout in second for distributed setting synchronization.
    """

    def __init__(
        self,
        task: str,
        resulting_name: Optional[str] = None,
        compute_kwargs: Optional[Dict[str, Any]] = None,
        config_name: Optional[str] = None,
        keep_in_memory: bool = False,
        cache_dir: Optional[str] = None,
        num_process: int = 1,
        process_id: int = 0,
        seed: Optional[int] = None,
        experiment_id: Optional[str] = None,
        max_concurrent_cache_files: int = 10000,
        timeout: Union[int, float] = 100,
        **kwargs,
    ):
        super().__init__(
            config_name=config_name,
            keep_in_memory=keep_in_memory,
            cache_dir=cache_dir,
            num_process=num_process,
            process_id=process_id,
            seed=seed,
            experiment_id=experiment_id,
            max_concurrent_cache_files=max_concurrent_cache_files,
            timeout=timeout,
            **kwargs,
        )
        self._task = task
        self.resulting_name = resulting_name if resulting_name is not None else self.name
        self.compute_kwargs = compute_kwargs or {}
        self.download_and_prepare()

    @abstractmethod
    def _compute(
        self,
        *,
        predictions: EvaluationInstance = None,
        references: EvaluationInstance = None,
        **kwargs,
    ) -> MetricOutput:
        """
        Base compute method which is used for internal computation. All child classes
        must implement _compute() method.
        """
        pass

    @abstractmethod
    def _compute_single_pred_single_ref(
        self,
        predictions: EvaluationInstance,
        references: EvaluationInstance,
        **kwargs,
    ):
        """
        Computes the metric score(s) for single prediction and single reference case.

        Args:
            predictions: (``EvaluationInstance``) Predictions
            references: (``EvaluationInstance``) References
            **kwargs: Additional arguments used for the metric computation.

        Returns: score
        """
        pass

    @abstractmethod
    def _compute_single_pred_multi_ref(
        self,
        predictions: EvaluationInstance,
        references: EvaluationInstance,
        **kwargs,
    ):
        """
        Computes the metric score(s) for single prediction and multiple references case.

        Args:
            predictions: (``EvaluationInstance``) Predictions
            references: (``EvaluationInstance``) References
            **kwargs: Additional arguments used for the metric computation.

        Returns: score
        """
        pass

    @abstractmethod
    def _compute_multi_pred_multi_ref(
        self,
        predictions: EvaluationInstance,
        references: EvaluationInstance,
        **kwargs,
    ):
        """
        Computes the metric score(s) for single prediction and multiple references case.

        Args:
            predictions: (``EvaluationInstance``) Predictions
            references: (``EvaluationInstance``) References
            **kwargs: Additional arguments used for the metric computation.

        Returns: score
        """
        pass

    def _download_and_prepare(self, dl_manager):
        """
        Downloads and prepares resources for the metric.

        This is the internal implementation to overwrite called when user calls
        `download_and_prepare`. It should download all required resources for the metric.

        Args:
            dl_manager (:class:`DownloadManager`): `DownloadManager` used to download and cache data.
        """
        self.external_module_path = None
        return None

    def _get_external_resource(self, module_name: Optional[str], attr: Optional[str] = None):
        if self.external_module_path is None:
            raise AttributeError("'external_module_path' is not defined.")
        if module_name is None:
            module_name = "external_module"
        external_module = import_module(module_name, self.external_module_path)
        if attr is None:
            return external_module
        return getattr(external_module, attr)

    @property
    def task(self):
        return self._task


class MetricForTask(Metric):
    """
    Base metric class for any task. All metrics must extend this class as metric is required to adopt a task
    inherently. Default task will be language-generation for AutoMetric.

    All metrics extending :py:class:`jury.metrics._core.base.MetricForTask` must implement the following:

        - _task (``[str]``): Task name for the base task metric.
        - _default_features() (``datasets.Features``): Task input as a :py:class:`datasets.Features`.

     Args:
        resulting_name (Optional ``[str]``): Optional resulting name for :py:class:`jury.Jury` to use. By default, it
            uses `metric.name` if not given. This is meant to prevent clashes for output dict of
            :py:method:`jury.Jury.evaluate` such as when bleu-1, and bleu-2 are used together.
        compute_kwargs (Optional ``Dict[str, Any]``): These are the parameters to be passed to compute function of the
            metric. It is meant to ease the support of computation from a jury configuration file, etc.
    """

    _task = None

    def __init__(self, resulting_name: Optional[str] = None, compute_kwargs: Optional[Dict[str, Any]] = None, **kwargs):
        compute_kwargs = self._validate_compute_kwargs(compute_kwargs)
        super().__init__(task=self._task, resulting_name=resulting_name, compute_kwargs=compute_kwargs, **kwargs)

    def _validate_compute_kwargs(self, compute_kwargs: Dict[str, Any]) -> Dict[str, Any]:
        if compute_kwargs is not None and "reduce_fn" in compute_kwargs:
            compute_kwargs.pop("reduce_fn")
        return compute_kwargs

    @property
    def _default_features(self):
        raise NotImplementedError

    @classmethod
    def _construct(
        cls, resulting_name: Optional[str] = None, compute_kwargs: Optional[Dict[str, Any]] = None, **kwargs
    ):
        return cls(resulting_name=resulting_name, compute_kwargs=compute_kwargs, **kwargs)

    @staticmethod
    def _reduce_scores(scores: Union[List[Dict[str, float]], List[float]], reduce_fn: Callable):
        if isinstance(scores[0], dict):
            score = pd.DataFrame(scores).apply(reduce_fn, axis=0).to_dict()
        else:
            score = float(reduce_fn(scores))
        return score

    def _preprocess(self, predictions: List[List[str]], references: List[List[str]]) -> Tuple[Collator, Collator]:
        return Collator(predictions, keep=True), Collator(references, keep=True)

    def _compute(
        self,
        *,
        predictions: EvaluationInstance = None,
        references: EvaluationInstance = None,
        **kwargs,
    ) -> MetricOutput:
        assert len(predictions) == len(references), "Predictions and references length does not match."
        eval_params = {**self.compute_kwargs, **kwargs}
        if "reduce_fn" in eval_params:
            eval_params.pop("reduce_fn")
        predictions, references = Collator(predictions), Collator(references)
        result = self.evaluate(predictions=predictions, references=references, **eval_params)
        return {self.resulting_name: result}

    def evaluate(self, predictions: Collator, references: Collator, **kwargs) -> Dict[str, float]:
        if predictions.can_collapse() and references.can_collapse():
            predictions = predictions.collapse()
            references = references.collapse()
            eval_fn = self._compute_single_pred_single_ref
        elif predictions.can_collapse() and not references.can_collapse():
            predictions = predictions.collapse()
            eval_fn = self._compute_single_pred_multi_ref
        else:
            eval_fn = self._compute_multi_pred_multi_ref

        predictions, references = self._preprocess(predictions, references)
        return eval_fn(predictions=predictions, references=references, **kwargs)


class MetricForLanguageGeneration(MetricForTask):
    """
    Base metric class for language generation task. Many metrics on jury are language generation metrics which are
    used by default by :py:class:`jury.metrics.AutoMetric`.
    """

    _task = "language-generation"

    @property
    def _default_features(self):
        return datasets.Features(
            {
                "predictions": datasets.Sequence(datasets.Value("string", id="sequence")),
                "references": datasets.Sequence(datasets.Value("string", id="sequence")),
            }
        )

    def _validate_compute_kwargs(self, compute_kwargs: Dict[str, Any]) -> Dict[str, Any]:
        if compute_kwargs is None:
            compute_kwargs = {}
        if "reduce_fn" not in compute_kwargs:
            compute_kwargs.update({"reduce_fn": "max"})
        return compute_kwargs

    def _compute(
        self,
        *,
        predictions: LanguageGenerationInstance = None,
        references: LanguageGenerationInstance = None,
        **kwargs,
    ) -> MetricOutput:
        assert len(predictions) == len(references), "Predictions and references length does not match."
        reduce_fn = kwargs.get("reduce_fn")
        reduce_fn = self.compute_kwargs["reduce_fn"] if reduce_fn is None else reduce_fn
        if isinstance(reduce_fn, str):
            reduce_fn = getattr(numpy, reduce_fn)
        elif reduce_fn is not None and not callable(reduce_fn):
            raise TypeError(f"'reduce_fn' Expected str or callable, got {type(reduce_fn)}")
        if reduce_fn is not None and not is_reduce_fn(reduce_fn):
            raise ValueError("'reduce_fn' must be an aggregation function.")
        eval_params = {**self.compute_kwargs, **kwargs}
        eval_params.pop("reduce_fn")
        predictions, references = Collator(predictions), Collator(references)
        result = self.evaluate(predictions=predictions, references=references, reduce_fn=reduce_fn, **eval_params)
        return {self.resulting_name: result}

    @abstractmethod
    def _compute_single_pred_single_ref(
        self,
        predictions: LanguageGenerationInstance,
        references: LanguageGenerationInstance,
        reduce_fn: Callable = None,
        **kwargs,
    ):
        """
        Computes the metric score(s) for single prediction and single reference case.

        Args:
            predictions: (``List[str]``) Predictions
            references: (``List[str]``) References
            reduce_fn: (``Callable``) Reduce function name.
            **kwargs: Additional arguments used for the metric computation.

        Returns: score
        """
        pass

    @abstractmethod
    def _compute_single_pred_multi_ref(
        self,
        predictions: LanguageGenerationInstance,
        references: LanguageGenerationInstance,
        reduce_fn: Callable = None,
        **kwargs,
    ):
        """
        Computes the metric score(s) for single prediction and multiple references case.

        Args:
            predictions: (``List[str]``) Predictions
            references: (``List[List[str]]``) References
            reduce_fn: (``Callable``) Reduce function name.
            **kwargs: Additional arguments used for the metric computation.

        Returns: score
        """
        pass

    @abstractmethod
    def _compute_multi_pred_multi_ref(
        self,
        predictions: LanguageGenerationInstance,
        references: LanguageGenerationInstance,
        reduce_fn: Callable = None,
        **kwargs,
    ):
        """
        Computes the metric score(s) for single prediction and multiple references case.

        Args:
            predictions: (``List[List[str]]``) Predictions
            references: (``List[List[str]]``) References
            reduce_fn: (``Callable``) Reduce function name.
            **kwargs: Additional arguments used for the metric computation.

        Returns: score
        """
        pass


class MetricForSequenceClassification(MetricForTask):
    """
    Base metric class for sequence classification task.
    """

    _task = "sequence-classification"

    @property
    def _default_features(self):
        return datasets.Features(
            {
                "predictions": datasets.Sequence(datasets.Value("int32")),
                "references": datasets.Sequence(datasets.Value("int32")),
            }
        )

    @abstractmethod
    def _compute_single_pred_single_ref(
        self,
        predictions: SequenceClassificationInstance,
        references: SequenceClassificationInstance,
        reduce_fn: Callable = None,
        **kwargs,
    ):
        """
        Computes the metric score(s) for single prediction and single reference case.

        Args:
            predictions: (``List[int]``) Predictions
            references: (``List[int]``) References
            **kwargs: Additional arguments used for the metric computation.

        Returns: score
        """
        pass

    @abstractmethod
    def _compute_single_pred_multi_ref(
        self,
        predictions: SequenceClassificationInstance,
        references: SequenceClassificationInstance,
        reduce_fn: Callable = None,
        **kwargs,
    ):
        pass

    @abstractmethod
    def _compute_multi_pred_multi_ref(
        self,
        predictions: SequenceClassificationInstance,
        references: SequenceClassificationInstance,
        reduce_fn: Callable = None,
        **kwargs,
    ):
        pass

    @staticmethod
    def _get_class_ids(references: List[List]):
        stacked_references = []
        for ref in references:
            stacked_references.extend(ref)
        return np.unique(stacked_references).tolist()


class MetricForSequenceLabeling(MetricForTask):
    """
    Base metric class for sequence labeling task.
    """

    _task = "sequence-labeling"

    @property
    def _default_features(self):
        return datasets.Features(
            {
                "predictions": datasets.Sequence(datasets.Value("string", id="sequence")),
                "references": datasets.Sequence(datasets.Value("string", id="sequence")),
            }
        )

    @abstractmethod
    def _compute_single_pred_single_ref(
        self,
        predictions: SequenceLabelingInstance,
        references: SequenceLabelingInstance,
        **kwargs,
    ):
        """
        Computes the metric score(s) for single prediction and single reference case.

        Args:
            predictions: (``List[List[str]]``) Predictions
            references: (``List[List[str]]``) References
            **kwargs: Additional arguments used for the metric computation.

        Returns: score
        """
        pass

    def _compute_single_pred_multi_ref(
        self,
        predictions: SequenceLabelingInstance,
        references: SequenceLabelingInstance,
        **kwargs,
    ):
        raise NotImplementedError(f"Task {self._task} does not support multiple predictions or multiple" "references.")

    def _compute_multi_pred_multi_ref(
        self,
        predictions: SequenceLabelingInstance,
        references: SequenceLabelingInstance,
        **kwargs,
    ):
        raise NotImplementedError(f"Task {self._task} does not support multiple predictions or multiple" "references.")



from typing import Any, Dict, Optional

from jury.metrics._core import MetricAlias
from jury.metrics.bleu.bleu_for_language_generation import BleuForLanguageGeneration

__main_class__ = "Bleu"


class Bleu(MetricAlias):
    _SUBCLASS = BleuForLanguageGeneration

    @classmethod
    def construct(
        cls,
        task: str = None,
        resulting_name: Optional[str] = None,
        compute_kwargs: Optional[Dict[str, Any]] = None,
        **kwargs,
    ):
        subclass = cls._get_subclass()
        resulting_name = resulting_name or cls._get_metric_name(compute_kwargs=compute_kwargs)
        return subclass._construct(resulting_name=resulting_name, compute_kwargs=compute_kwargs, **kwargs)

    @classmethod
    def _get_metric_name(cls, compute_kwargs: Dict[str, Any] = None) -> str:
        """
        All metric modules must implement this method as it is used to form MetricOutput properly.

        Returns: Metric name.
        """
        metric_name = cls.__name__.lower()
        if compute_kwargs is None:
            return metric_name

        max_order = compute_kwargs.get("max_order")
        if max_order is not None:
            return f"{metric_name}_{max_order}"



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
F1 metric. The part of this file is adapted from HuggingFace's
datasets package implementation of F1 metric. See
https://github.com/huggingface/datasets/blob/master/metrics/f1/f1.py
"""
from typing import List, Union

import datasets
from sklearn.metrics import f1_score

from jury.metrics._core import MetricForSequenceClassification, SequenceClassificationInstance, load_metric

_DESCRIPTION = """
The F1 score is the harmonic mean of the precision and recall. It can be computed with:
F1 = 2 * (precision * recall) / (precision + recall)
"""

_KWARGS_DESCRIPTION = """
Args:
    predictions: Predicted labels, as returned by a model.
    references: Ground truth labels.
    labels: The set of labels to include when average != 'binary', and
        their order if average is None. Labels present in the data can
        be excluded, for example to calculate a multiclass average ignoring
        a majority negative class, while labels not present in the data will
        result in 0 components in a macro average. For multilabel targets,
        labels are column indices. By default, all labels in y_true and
        y_pred are used in sorted order.
    average: This parameter is required for multiclass/multilabel targets.
        If None, the scores for each class are returned. Otherwise, this
        determines the type of averaging performed on the data:
            binary: Only report results for the class specified by pos_label.
                This is applicable only if targets (y_{true,pred}) are binary.
            micro: Calculate metrics globally by counting the total true positives,
                false negatives and false positives.
            macro: Calculate metrics for each label, and find their unweighted mean.
                This does not take label imbalance into account.
            weighted: Calculate metrics for each label, and find their average
                weighted by support (the number of true instances for each label).
                This alters ‘macro’ to account for label imbalance; it can result
                in an F-score that is not between precision and recall.
            samples: Calculate metrics for each instance, and find their average
                (only meaningful for multilabel classification).
    sample_weight: Sample weights.
Returns:
    f1: F1 score.
Examples:

    >>> f1_metric = datasets.load_metric("f1")
    >>> results = f1_metric.compute(predictions=[0, 1], references=[0, 1])
    >>> print(results)
    {'f1': 1.0}

    >>> predictions = [0, 2, 1, 0, 0, 1]
    >>> references = [0, 1, 2, 0, 1, 2]
    >>> results = f1_metric.compute(predictions=predictions, references=references, average="macro")
    >>> print(results)
    {'f1': 0.26666666666666666}
    >>> results = f1_metric.compute(predictions=predictions, references=references, average="micro")
    >>> print(results)
    {'f1': 0.3333333333333333}
    >>> results = f1_metric.compute(predictions=predictions, references=references, average="weighted")
    >>> print(results)
    {'f1': 0.26666666666666666}
    >>> results = f1_metric.compute(predictions=predictions, references=references, average=None)
    >>> print(results)
    {'f1': array([0.8, 0. , 0. ])}
"""

_CITATION = """
@article{scikit-learn,
  title={Scikit-learn: Machine Learning in {P}ython},
  author={Pedregosa, F. and Varoquaux, G. and Gramfort, A. and Michel, V.
         and Thirion, B. and Grisel, O. and Blondel, M. and Prettenhofer, P.
         and Weiss, R. and Dubourg, V. and Vanderplas, J. and Passos, A. and
         Cournapeau, D. and Brucher, M. and Perrot, M. and Duchesnay, E.},
  journal={Journal of Machine Learning Research},
  volume={12},
  pages={2825--2830},
  year={2011}
}
"""


@datasets.utils.file_utils.add_start_docstrings(_DESCRIPTION, _KWARGS_DESCRIPTION)
class F1ForSequenceClassification(MetricForSequenceClassification):
    _precision = load_metric("precision", task="sequence-classification")
    _recall = load_metric("recall", task="sequence-classification")

    def _info(self):
        return datasets.MetricInfo(
            description=_DESCRIPTION,
            citation=_CITATION,
            inputs_description=_KWARGS_DESCRIPTION,
            features=self._default_features,
            reference_urls=["https://scikit-learn.org/stable/modules/generated/sklearn.metrics.precision_score.html"],
        )

    def _compute_single_pred_single_ref(
        self,
        predictions: SequenceClassificationInstance,
        references: SequenceClassificationInstance,
        labels=None,
        pos_label=1,
        average=None,
        sample_weight=None,
    ):
        score = f1_score(
            references, predictions, labels=labels, pos_label=pos_label, average=average, sample_weight=sample_weight
        )
        return {"score": float(score) if score.size == 1 else score.tolist()}

    def _compute_single_pred_multi_ref(
        self, predictions: SequenceClassificationInstance, references: SequenceClassificationInstance, **kwargs
    ):
        predictions = predictions.nested()
        return self._compute_multi_pred_multi_ref(predictions=predictions, references=references, **kwargs)

    def _compute_multi_pred_multi_ref(
        self, predictions: SequenceClassificationInstance, references: SequenceClassificationInstance, **kwargs
    ):
        precision = self._precision.compute(predictions=predictions, references=references, **kwargs)
        recall = self._recall.compute(predictions=predictions, references=references, **kwargs)
        precision_scores = precision["precision"]["score"]
        recall_scores = recall["recall"]["score"]
        f1 = self._compute_f1(precision_scores, recall_scores)
        return {"score": f1}

    @staticmethod
    def _harmonic_mean(p, r):
        return 2 * p * r / (p + r)

    def _compute_f1(
        self, precision: Union[float, List[float]], recall: Union[float, List[float]]
    ) -> Union[float, List[float]]:
        if isinstance(precision, list) and isinstance(recall, list):
            return [self._harmonic_mean(p, r) for p, r in zip(precision, recall)]

        return self._harmonic_mean(precision, recall)



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Modified Unigram Precision metric. The part of this file is adapted from HuggingFace's
datasets package implementation of Precision metric. See
https://github.com/huggingface/datasets/blob/master/metrics/precision/precision.py
"""
from collections import Counter
from typing import Callable

import datasets
import numpy as np

from jury.collator import Collator
from jury.metrics._core import MetricForLanguageGeneration
from jury.metrics._core.utils import normalize_text

_CITATION = """
    @inproceedings{papineni2002bleu,
      title={Bleu: a method for automatic evaluation of machine translation},
      author={Papineni, Kishore and Roukos, Salim and Ward, Todd and Zhu, Wei-Jing},
      booktitle={Proceedings of the 40th annual meeting of the Association for Computational Linguistics},
      pages={311--318},
      year={2002}
    }
    """

_DESCRIPTION = """
Modified Unigram Precision is the fraction of the common unigrams between the prediction
and the references among the prediction tokens. It can be computed with:
Precision = # of matching tokens / # of prediction tokens
"""

_KWARGS_DESCRIPTION = """
Args:
    predictions: list of predictions to score. Each predictions
        should be a string with tokens separated by spaces.
    references: list of reference for each prediction. Each
        reference should be a string with tokens separated by spaces.
    Returns:
    'score': Precision score.
    Examples:
    
    >>> precision = jury.load_metric("precision")
    >>> predictions = [["the cat is on the mat", "There is cat playing on the mat"], ["Look! a wonderful day."]]
    >>> references = [
        ["the cat is playing on the mat.", "The cat plays on the mat."], 
        ["Today is a wonderful day", "The weather outside is wonderful."]
    ]
    >>> results = precision.compute(predictions=predictions, references=references)
    >>> print(results)
    {'precision': {'score': 0.875}}
"""


@datasets.utils.file_utils.add_start_docstrings(_DESCRIPTION, _KWARGS_DESCRIPTION)
class PrecisionForLanguageGeneration(MetricForLanguageGeneration):
    def _info(self):
        return datasets.MetricInfo(
            description=_DESCRIPTION,
            citation=_CITATION,
            inputs_description=_KWARGS_DESCRIPTION,
            features=self._default_features,
            reference_urls=["https://scikit-learn.org/stable/modules/generated/sklearn.metrics.precision_score.html"],
        )

    def _tokenize(self, predictions: Collator, references: Collator):
        predictions = [normalize_text(p).split() for p in predictions]
        references = [normalize_text(r).split() for r in references]
        return predictions, references

    def _compute_single_pred_single_ref(
        self, predictions: Collator, references: Collator, reduce_fn: Callable = None, **kwargs
    ):
        scores = []
        predictions, references = self._tokenize(predictions, references)
        for pred, ref in zip(predictions, references):
            score = 0
            pred_counts = Counter(pred)
            ref_counts = Counter(ref)
            for token, pred_count in pred_counts.items():
                if token in ref_counts:
                    score += min(pred_count, ref_counts[token])  # Intersection count
            scores.append(score / len(pred))
        avg_score = sum(scores) / len(scores)
        return {"score": avg_score}

    def _compute_single_pred_multi_ref(
        self, predictions: Collator, references: Collator, reduce_fn: Callable = None, **kwargs
    ):
        scores = []
        for pred, refs in zip(predictions, references):
            pred_score = [
                self._compute_single_pred_single_ref(Collator([pred], keep=True), Collator([ref], keep=True))
                for ref in refs
            ]
            reduced_score = self._reduce_scores(pred_score, reduce_fn=reduce_fn)
            scores.append(reduced_score)

        return self._reduce_scores(scores, reduce_fn=np.mean)

    def _compute_multi_pred_multi_ref(
        self, predictions: Collator, references: Collator, reduce_fn: Callable = None, **kwargs
    ):
        scores = []
        for preds, refs in zip(predictions, references):
            pred_scores = []
            for pred in preds:
                pred_score = self._compute_single_pred_multi_ref(
                    Collator([pred], keep=True), Collator([refs], keep=True), reduce_fn=reduce_fn
                )
                pred_scores.append(pred_score)
            reduced_score = self._reduce_scores(pred_scores, reduce_fn=reduce_fn)
            scores.append(reduced_score)

        return self._reduce_scores(scores, reduce_fn=np.mean)



# coding=utf-8
# Copyright 2021 Open Business Software Solutions, The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" BERTScore metric. The part of this file is adapted from BERTScore implementation
of datasets package. See
https://github.com/huggingface/datasets/blob/master/metrics/bertscore/bertscore.py """

import functools
from contextlib import contextmanager
from typing import Dict, List

import datasets
import numpy as np
import pandas as pd
from packaging import version

from jury.collator import Collator
from jury.metrics._core import MetricForLanguageGeneration
from jury.metrics._core.utils import PackagePlaceholder, requirement_message

# `import bert_score` placeholder
bert_score = PackagePlaceholder(version="0.3.10")


@contextmanager
def filter_logging_context():
    def filter_log(record):
        return False if "This IS expected if you are initializing" in record.msg else True

    logger = datasets.utils.logging.get_logger("transformers.modeling_utils")
    logger.addFilter(filter_log)
    try:
        yield
    finally:
        logger.removeFilter(filter_log)


_CITATION = """\
@inproceedings{bert-score,
  title={BERTScore: Evaluating Text Generation with BERT},
  author={Tianyi Zhang* and Varsha Kishore* and Felix Wu* and Kilian Q. Weinberger and Yoav Artzi},
  booktitle={International Conference on Learning Representations},
  year={2020},
  url={https://openreview.net/forum?id=SkeHuCVFDr}
}
"""

_DESCRIPTION = """\
BERTScore leverages the pre-trained contextual embeddings from BERT and matches words in candidate and reference
sentences by cosine similarity.
It has been shown to correlate with human judgment on sentence-level and system-level evaluation.
Moreover, BERTScore computes precision, recall, and F1 measure, which can be useful for evaluating different language
generation tasks.

See the `README.md` file at [https://github.com/Tiiiger/bert_score](https://github.com/Tiiiger/bert_score) for more
information.
"""

_KWARGS_DESCRIPTION = """
BERTScore Metrics with the hashcode from a source against one or more references.

Args:
    predictions (list of str): Prediction/candidate sentences.
    references (list of str or list of list of str): Reference sentences.
    lang (str): Language of the sentences; required (e.g. 'en').
    model_type (str): Bert specification, default using the suggested
        model for the target language; has to specify at least one of
        `model_type` or `lang`.
    num_layers (int): The layer of representation to use,
        default using the number of layers tuned on WMT16 correlation data.
    verbose (bool): Turn on intermediate status update.
    idf (bool or dict): Use idf weighting; can also be a precomputed idf_dict.
    device (str): On which the contextual embedding model will be allocated on.
        If this argument is None, the model lives on cuda:0 if cuda is available.
    nthreads (int): Number of threads.
    batch_size (int): Bert score processing batch size,
        at least one of `model_type` or `lang`. `lang` needs to be
        specified when `rescale_with_baseline` is True.
    rescale_with_baseline (bool): Rescale bertscore with pre-computed baseline.
    baseline_path (str): Customized baseline file.
    use_fast_tokenizer (bool): `use_fast` parameter passed to HF tokenizer. New in version 0.3.10.

Returns:
    'score': Bertscore f1. This is always the same as 'f1' in cases single-prediction and single-reference, and
        single-prediction and multiple-references, otherwise it is reduced version of 'f1' by `reduce_fn`. 
    'precision': Precision.
    'recall': Recall.
    'f1': F1 score.
    'hashcode': Hashcode of the library.

Examples:

    >>> bertscore = jury.load_metric("bertscore")
    >>> predictions = [["the cat is on the mat", "There is cat playing on the mat"], ["Look! a wonderful day."]]
    >>> references = [
        ["the cat is playing on the mat.", "The cat plays on the mat."], 
        ["Today is a wonderful day", "The weather outside is wonderful."]
    ]
    >>> results = bertscore.compute(predictions=predictions, references=references)
    >>> print(results)
    {'bertscore': {'score': 0.9473764896392822, 'precision': 0.9467198252677917, 'recall': 0.9480386078357697, 
        'f1': 0.9473764896392822, 'hashcode': 'roberta-large_L17_no-idf_version=0.3.10(hug_trans=4.9.1)'}}
"""


@datasets.utils.file_utils.add_start_docstrings(_DESCRIPTION, _KWARGS_DESCRIPTION)
class BertscoreForLanguageGeneration(MetricForLanguageGeneration):
    def _download_and_prepare(self, dl_manager):
        global bert_score

        try:
            import bert_score
        except ModuleNotFoundError:
            raise ModuleNotFoundError(requirement_message(metric_name="Bertscore", package_name="bert-score"))
        else:
            super(BertscoreForLanguageGeneration, self)._download_and_prepare(dl_manager)

    def _info(self):
        return datasets.MetricInfo(
            description=_DESCRIPTION,
            citation=_CITATION,
            homepage="https://github.com/Tiiiger/bert_score",
            inputs_description=_KWARGS_DESCRIPTION,
            features=self._default_features,
            codebase_urls=["https://github.com/Tiiiger/bert_score"],
            reference_urls=[
                "https://github.com/Tiiiger/bert_score",
                "https://arxiv.org/abs/1904.09675",
            ],
        )

    def _compute_single_pred_single_ref(
        self,
        predictions,
        references,
        reduce_fn=None,
        lang="en",
        model_type=None,
        num_layers=None,
        verbose=False,
        idf=False,
        device=None,
        batch_size=64,
        nthreads=4,
        all_layers=False,
        rescale_with_baseline=False,
        baseline_path=None,
        use_fast_tokenizer=False,
        return_average_scores=False,
    ):
        get_hash = bert_score.utils.get_hash
        scorer = bert_score.BERTScorer

        if version.parse(bert_score.__version__) >= version.parse("0.3.10"):
            get_hash = functools.partial(get_hash, use_fast_tokenizer=use_fast_tokenizer)
            scorer = functools.partial(scorer, use_fast_tokenizer=use_fast_tokenizer)
        elif use_fast_tokenizer:
            raise ImportWarning(
                "To use a fast tokenizer, the module `bert-score>=0.3.10` is required, and the current version of `bert-score` doesn't match this condition.\n"
                'You can install it with `pip install "bert-score>=0.3.10"`.'
            )

        if model_type is None:
            assert lang is not None, "either lang or model_type should be specified"
            model_type = bert_score.utils.lang2model[lang.lower()]

        if num_layers is None:
            num_layers = bert_score.utils.model2layers[model_type]

        hashcode = get_hash(
            model=model_type,
            num_layers=num_layers,
            idf=idf,
            rescale_with_baseline=rescale_with_baseline,
            use_custom_baseline=baseline_path is not None,
        )

        with filter_logging_context():
            if not hasattr(self, "cached_bertscorer") or self.cached_bertscorer.hash != hashcode:
                self.cached_bertscorer = scorer(
                    model_type=model_type,
                    num_layers=num_layers,
                    batch_size=batch_size,
                    nthreads=nthreads,
                    all_layers=all_layers,
                    idf=idf,
                    device=device,
                    lang=lang,
                    rescale_with_baseline=rescale_with_baseline,
                    baseline_path=baseline_path,
                )

        (P, R, F) = self.cached_bertscorer.score(
            cands=predictions,
            refs=references,
            verbose=verbose,
            batch_size=batch_size,
        )

        P = P.tolist()
        R = R.tolist()
        F = F.tolist()
        score = float(np.mean(F))

        if return_average_scores:
            P = float(np.mean(P))
            R = float(np.mean(R))
            F = float(np.mean(F))

        output_dict = {
            "score": score,
            "precision": P,
            "recall": R,
            "f1": F,
            "hashcode": hashcode,
        }
        return output_dict

    def _compute_single_pred_multi_ref(
        self,
        predictions,
        references,
        reduce_fn=None,
        lang="en",
        model_type=None,
        num_layers=None,
        verbose=False,
        idf=False,
        device=None,
        batch_size=64,
        nthreads=4,
        all_layers=False,
        rescale_with_baseline=False,
        baseline_path=None,
        use_fast_tokenizer=False,
        return_average_scores=False,
    ):
        # BERTScore inherently supports multiple references
        return self._compute_single_pred_single_ref(
            predictions=predictions,
            references=references,
            reduce_fn=reduce_fn,
            lang=lang,
            model_type=model_type,
            num_layers=num_layers,
            verbose=verbose,
            idf=idf,
            device=device,
            batch_size=batch_size,
            nthreads=nthreads,
            all_layers=all_layers,
            rescale_with_baseline=rescale_with_baseline,
            baseline_path=baseline_path,
            use_fast_tokenizer=use_fast_tokenizer,
            return_average_scores=return_average_scores,
        )

    def _compute_multi_pred_multi_ref(
        self,
        predictions,
        references,
        reduce_fn=None,
        lang="en",
        model_type=None,
        num_layers=None,
        verbose=False,
        idf=False,
        device=None,
        batch_size=64,
        nthreads=4,
        all_layers=False,
        rescale_with_baseline=False,
        baseline_path=None,
        use_fast_tokenizer=False,
    ):
        scores = []
        for preds, refs in zip(predictions, references):
            pred_scores = [
                self._compute_single_pred_multi_ref(
                    predictions=Collator([pred], keep=True),
                    references=Collator([refs], keep=True),
                    reduce_fn=reduce_fn,
                    lang=lang,
                    model_type=model_type,
                    num_layers=num_layers,
                    verbose=verbose,
                    idf=idf,
                    device=device,
                    batch_size=batch_size,
                    nthreads=nthreads,
                    all_layers=all_layers,
                    rescale_with_baseline=rescale_with_baseline,
                    baseline_path=baseline_path,
                    use_fast_tokenizer=use_fast_tokenizer,
                    return_average_scores=True,
                )
                for pred in preds
            ]
            hashcode = pred_scores[0]["hashcode"]
            reduced_score = self._reduce_multi_pred_scores(pred_scores, reduce_fn=reduce_fn)
            scores.append(reduced_score)

        # Average reduced scores
        return self._reduce_multi_pred_scores(scores, reduce_fn=np.mean, hashcode=hashcode)

    def _reduce_multi_pred_scores(self, results: List[Dict], reduce_fn, **kwargs) -> Dict:
        df = pd.DataFrame(results)
        if "hashcode" in df:
            df.drop("hashcode", axis=1, inplace=True)
        scores = df.apply(reduce_fn, axis=0).to_dict()
        scores.update(kwargs)
        return scores

    def add_batch(self, predictions=None, references=None, **kwargs):
        """Add a batch of predictions and references for the metric's stack."""
        # References can be strings or lists of strings
        # Let's change strings to lists of strings with one element
        if references is not None:
            references = [[ref] if isinstance(ref, str) else ref for ref in references]
        super().add_batch(predictions=predictions, references=references, **kwargs)

    def add(self, prediction=None, reference=None, **kwargs):
        """Add one prediction and reference for the metric's stack."""
        # References can be strings or lists of strings
        # Let's change strings to lists of strings with one element
        if isinstance(reference, str):
            reference = [reference]
        super().add(prediction=prediction, reference=reference, **kwargs)



from jury.metrics._core import MetricAlias
from jury.metrics.meteor.meteor_for_language_generation import MeteorForLanguageGeneration

__main_class__ = "Meteor"


class Meteor(MetricAlias):
    _SUBCLASS = MeteorForLanguageGeneration



import pytest

from jury import Jury
from jury.metrics import AutoMetric
from tests.jury.conftest import get_expected_output
from tests.utils import assert_almost_equal_dict


@pytest.fixture(scope="module")
def jury():
    metric = AutoMetric.load("seqeval")
    return Jury(metrics=metric)


@pytest.fixture
def predictions():
    return [["O", "O", "B-MISC", "I-MISC", "I-MISC", "I-MISC", "O"], ["B-PER", "I-PER", "O"]]


@pytest.fixture
def references():
    return [["O", "O", "O", "B-MISC", "I-MISC", "I-MISC", "O"], ["B-PER", "I-PER", "O"]]


@pytest.fixture
@get_expected_output(prefix="metrics")
def output_basic():
    return output_basic.output


def test_basic(predictions, references, jury, output_basic):
    scores = jury(predictions=predictions, references=references)
    assert_almost_equal_dict(actual=scores, desired=output_basic)



# coding=utf-8
# Copyright 2020 Open Business Software Solutions, The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
ROUGE metric. The part of this file is adapted from HuggingFace's
datasets package implementation of ROUGE metric. See
https://github.com/huggingface/datasets/blob/master/metrics/rouge/rouge.py
"""

from typing import Callable, Dict, List, Optional, Union

import datasets
import pandas as pd
from rouge_score import rouge_scorer, scoring

from jury.collator import Collator
from jury.metrics._core import MetricForLanguageGeneration

_CITATION = """\
@inproceedings{lin-2004-rouge,
    title = "{ROUGE}: A Package for Automatic Evaluation of Summaries",
    author = "Lin, Chin-Yew",
    booktitle = "Text Summarization Branches Out",
    month = jul,
    year = "2004",
    address = "Barcelona, Spain",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/W04-1013",
    pages = "74--81",
}
"""

_DESCRIPTION = """\
ROUGE, or Recall-Oriented Understudy for Gisting Evaluation, is a set of metrics and a software package used for
evaluating automatic summarization and machine translation software in natural language processing.
The metrics compare an automatically produced summary or translation against a reference or a set of references 
(human-produced) summary or translation.

Note that ROUGE is case insensitive, meaning that upper case letters are treated the same way as lower case letters.

This metrics is a wrapper around Google Research reimplementation of ROUGE:
https://github.com/google-research/google-research/tree/master/rouge
"""

_KWARGS_DESCRIPTION = """
Calculates average rouge scores for a list of hypotheses and references
Args:
    predictions: list of predictions to score. Each predictions
        should be a string with tokens separated by spaces.
    references: list of reference for each prediction. Each
        reference should be a string with tokens separated by spaces.
    rouge_types: A list of rouge types to calculate.
        Valid names:
        `"rouge{n}"` (e.g. `"rouge1"`, `"rouge2"`) where: {n} is the n-gram based scoring,
        `"rougeL"`: Longest common subsequence based scoring.
        `"rougeLSum"`: rougeLsum splits text using `"\n"`.
        See details in https://github.com/huggingface/datasets/issues/617
    use_stemmer: Bool indicating whether Porter stemmer should be used to strip word suffixes.
    use_agregator: Return aggregates if this is set to True
Returns:
    rouge1: rouge_1 (precision, recall, f1),
    rouge2: rouge_2 (precision, recall, f1),
    rougeL: rouge_l (precision, recall, f1),
    rougeLsum: rouge_lsum (precision, recall, f1)
Examples:

    >>> rouge = jury.load_metric("rouge")
    >>> predictions = [["the cat is on the mat", "There is cat playing on the mat"], ["Look! a wonderful day."]]
    >>> references = [
        ["the cat is playing on the mat.", "The cat plays on the mat."], 
        ["Today is a wonderful day", "The weather outside is wonderful."]
    ]
    >>> results = rouge.compute(predictions=predictions, references=references)
    >>> print(results)
    {'rouge': {'rouge1': 0.7783882783882783, 'rouge2': 0.5925324675324675, 
        'rougeL': 0.7426739926739926, 'rougeLsum': 0.7426739926739926}}
"""


@datasets.utils.file_utils.add_start_docstrings(_DESCRIPTION, _KWARGS_DESCRIPTION)
class RougeForLanguageGeneration(MetricForLanguageGeneration):
    def _info(self):
        return datasets.MetricInfo(
            description=_DESCRIPTION,
            citation=_CITATION,
            inputs_description=_KWARGS_DESCRIPTION,
            features=self._default_features,
            codebase_urls=["https://github.com/google-research/google-research/tree/master/rouge"],
            reference_urls=[
                "https://en.wikipedia.org/wiki/ROUGE_(metric)",
                "https://github.com/google-research/google-research/tree/master/rouge",
            ],
        )

    @staticmethod
    def _get_aggregator(use_aggregator: bool) -> Union[List, scoring.BootstrapAggregator]:
        if use_aggregator:
            aggregator = scoring.BootstrapAggregator()
        else:
            aggregator = []
        return aggregator

    @staticmethod
    def _add_score(
        aggregator: Union[List, scoring.BootstrapAggregator],
        score: Union[Dict[str, float], Dict[str, scoring.BootstrapAggregator]],
    ) -> Union[List, scoring.BootstrapAggregator]:
        if isinstance(aggregator, scoring.BootstrapAggregator):
            aggregator.add_scores(score)
        else:
            aggregator.append(score)
        return aggregator

    @staticmethod
    def _aggregate(aggregator) -> Union[Dict[str, float], Dict[str, scoring.AggregateScore]]:
        if isinstance(aggregator, scoring.BootstrapAggregator):
            result = aggregator.aggregate()
        else:
            result = {}
            for key in aggregator[0]:
                result[key] = list(score[key] for score in aggregator)
        return result

    @staticmethod
    def _normalize_score_list(
        score_list: List[Dict[str, scoring.BootstrapAggregator]], metric_to_select: str
    ) -> List[Dict[str, scoring.BootstrapAggregator]]:
        for score_dict in score_list:
            for metric, score in score_dict.items():
                score_dict[metric] = getattr(score, metric_to_select)
        return score_list

    @staticmethod
    def _reduce_dict(score_list: List[Dict[str, scoring.BootstrapAggregator]], reduce_fn: callable) -> Dict[str, float]:
        return pd.DataFrame(score_list).apply(reduce_fn, axis=0).to_dict()

    @staticmethod
    def _select_mid_from_aggregation(aggregated_scores: Dict[str, scoring.AggregateScore]) -> Dict[str, float]:
        return {metric: score.mid for metric, score in aggregated_scores.items()}

    def evaluate(
        self,
        *,
        predictions: Collator = None,
        references: Collator = None,
        reduce_fn: Callable = None,
        rouge_types: List[str] = None,
        use_aggregator: bool = True,
        use_stemmer: bool = False,
        metric_to_select: Optional[str] = "fmeasure",
        **kwargs
    ):
        if rouge_types is None:
            rouge_types = ["rouge1", "rouge2", "rougeL", "rougeLsum"]

        scorer = rouge_scorer.RougeScorer(rouge_types=rouge_types, use_stemmer=use_stemmer)

        if predictions.can_collapse() and references.can_collapse():
            aggregator = self._get_aggregator(use_aggregator)
            predictions = predictions.collapse()
            references = references.collapse()
            aggregator = self._compute_single_pred_single_ref(
                predictions, references, reduce_fn, scorer, aggregator, metric_to_select
            )
        elif predictions.can_collapse() and not references.can_collapse():
            # Force to use BootstrapAggregator
            aggregator = self._get_aggregator(use_aggregator=True)
            predictions = predictions.collapse()
            aggregator = self._compute_single_pred_multi_ref(
                predictions, references, reduce_fn, scorer, aggregator, metric_to_select
            )
        else:
            # Force to use BootstrapAggregator
            aggregator = self._get_aggregator(use_aggregator=True)
            aggregator = self._compute_multi_pred_multi_ref(
                predictions, references, reduce_fn, scorer, aggregator, metric_to_select
            )

        result = self._aggregate(aggregator)

        if metric_to_select and use_aggregator:
            result = self._select_mid_from_aggregation(result)
        elif metric_to_select and not use_aggregator:
            result = {k: v[0] for k, v in result.items()}

        return result

    def _compute_single_pred_single_ref(
        self, predictions, references, reduce_fn: Callable = None, scorer=None, aggregator=None, metric_to_select=None
    ):
        for ref, pred in zip(references, predictions):
            score = scorer.score(target=ref, prediction=pred)
            if metric_to_select is not None:
                score = self._normalize_score_list(score_list=[score], metric_to_select=metric_to_select)[0]
            aggregator = self._add_score(aggregator, score)
        return aggregator

    def _compute_single_pred_multi_ref(
        self, predictions, references, reduce_fn: Callable = None, scorer=None, aggregator=None, metric_to_select=None
    ):
        for pred, refs in zip(predictions, references):
            pred_scores = [scorer.score(target=ref, prediction=pred) for ref in refs]
            pred_scores = self._normalize_score_list(pred_scores, metric_to_select)
            score = self._reduce_dict(pred_scores, reduce_fn)
            aggregator = self._add_score(aggregator, score)
        return aggregator

    def _compute_multi_pred_multi_ref(
        self, predictions, references, reduce_fn: Callable = None, scorer=None, aggregator=None, metric_to_select=None
    ):
        for preds, refs in zip(predictions, references):
            multi_aggregator = self._get_aggregator(use_aggregator=True)
            for pred in preds:
                pred_score = self._normalize_score_list(
                    [scorer.score(target=ref, prediction=pred) for ref in refs], metric_to_select
                )
                pred_score = self._reduce_dict(pred_score, reduce_fn)
                multi_aggregator = self._add_score(multi_aggregator, pred_score)
            score = self._select_mid_from_aggregation(self._aggregate(multi_aggregator))
            aggregator = self._add_score(aggregator, score)
        return aggregator



import json
from typing import List, Optional, Union

import fire
import pandas as pd

from jury import Jury
from jury import __version__ as jury_version


def file_extension(path: str) -> str:
    return ".".join(path.split("/")[-1].split(".")[1:])


def read_config(path: str):
    with open(path, "r") as jf:
        return json.load(jf)


def read_file(filepath: str) -> Union[List[str], List[List[str]]]:
    if file_extension(filepath) == "csv":
        df = pd.read_csv(filepath, header=None)
        content = df.to_numpy().tolist()
    elif file_extension(filepath) == "tsv":
        df = pd.read_csv(filepath, header=None)
        content = df.to_numpy().tolist()
    else:
        with open(filepath, "r") as in_file:
            content = in_file.readlines()
    return content


def from_file(predictions: str, references: str, reduce_fn: Optional[str] = None, config: Optional[str] = None):
    args = read_config(config) if config is not None else {}
    predictions = predictions if predictions is not None else args.get("predictions")
    references = references if references is not None else args.get("references")
    reduce_fn = reduce_fn if reduce_fn is not None else args.get("reduce_fn")
    metrics = args.get("metrics")

    predictions = read_file(predictions)
    references = read_file(references)
    jury = Jury(metrics=metrics)
    scores = jury(predictions=predictions, references=references, reduce_fn=reduce_fn)
    print(json.dumps(scores, default=str, indent=4))


def app() -> None:
    """Cli app."""
    fire.Fire({"version": jury_version, "eval": from_file})


if __name__ == "__main__":
    app()



from jury.metrics._core import MetricAlias
from jury.metrics.rouge.rouge_for_language_generation import RougeForLanguageGeneration

__main_class__ = "Rouge"


class Rouge(MetricAlias):
    _SUBCLASS = RougeForLanguageGeneration



import datasets
import numpy as np
import pytest

from tests.jury.conftest import get_expected_output
from tests.utils import assert_almost_equal_dict


@pytest.fixture
@get_expected_output(prefix=None)
def output_evaluate_concurrent():
    return output_evaluate_concurrent.output


@pytest.fixture
@get_expected_output(prefix=None)
def output_evaluate():
    return output_evaluate.output


@pytest.fixture
@get_expected_output(prefix=None)
def output_evaluate_str_input():
    return output_evaluate_str_input.output


@pytest.fixture
@get_expected_output(prefix=None)
def output_evaluate_list_str_input():
    return output_evaluate_list_str_input.output


@pytest.fixture
@get_expected_output(prefix=None)
def output_evaluate_list_dict_input():
    return output_evaluate_list_dict_input.output


@pytest.fixture
@get_expected_output(prefix=None)
def output_evaluate_list_mixed_input():
    return output_evaluate_list_mixed_input.output


@pytest.fixture
@get_expected_output(prefix=None)
def output_evaluate_datasets_metric():
    return output_evaluate_datasets_metric.output


@pytest.fixture
@get_expected_output(prefix=None)
def output_evaluate_corpus():
    return output_evaluate_corpus.output


@pytest.fixture
@get_expected_output(prefix=None)
def output_evaluate_multiple_predictions():
    return output_evaluate_multiple_predictions.output


@pytest.fixture
@get_expected_output(prefix=None)
def output_evaluate_sequence_classification():
    return output_evaluate_sequence_classification.output


def test_evaluate_concurrent(predictions, references, jury_concurrent, output_evaluate_concurrent):
    scores = jury_concurrent(predictions=predictions, references=references)
    assert_almost_equal_dict(
        actual=scores, desired=output_evaluate_concurrent, exclude_paths="root['bertscore']['hashcode']"
    )


def test_evaluate_no_input(predictions, references, jury):
    with pytest.raises(TypeError):
        jury(predictions=predictions)
        jury(references=references)
        jury()


def test_evaluate_inconsistent_input(inconsistent_predictions, references, jury):
    # Different length
    with pytest.raises(ValueError):
        jury(predictions=inconsistent_predictions, references=references)


def test_evaluate_inconsistent_tasks(predictions, references, jury):
    with pytest.raises(ValueError):
        jury.add_metric("seqeval")
    jury.remove_metric("seqeval")


def test_evaluate(predictions, references, jury, output_evaluate):
    scores = jury(predictions=predictions, references=references)
    assert_almost_equal_dict(actual=scores, desired=output_evaluate, exclude_paths="root['bertscore']['hashcode']")


def test_evaluate_str_input(predictions, references, jury_str, output_evaluate_str_input):
    scores = jury_str(predictions=predictions, references=references)
    assert_almost_equal_dict(actual=scores, desired=output_evaluate_str_input)


def test_evaluate_list_str_input(predictions, references, jury_list_str, output_evaluate_list_str_input):
    scores = jury_list_str(predictions=predictions, references=references)
    assert_almost_equal_dict(
        actual=scores, desired=output_evaluate_list_str_input, exclude_paths="root['bertscore']['hashcode']"
    )


def test_evaluate_list_dict_input(predictions, references, jury_list_dict, output_evaluate_list_dict_input):
    scores = jury_list_dict(predictions=predictions, references=references)
    assert_almost_equal_dict(
        actual=scores, desired=output_evaluate_list_dict_input, exclude_paths="root['bertscore']['hashcode']"
    )


def test_evaluate_list_mixed_input(predictions, references, jury_list_mixed, output_evaluate_list_mixed_input):
    scores = jury_list_mixed(predictions=predictions, references=references)
    assert_almost_equal_dict(
        actual=scores, desired=output_evaluate_list_mixed_input, exclude_paths="root['bertscore']['hashcode']"
    )


def test_evaluate_datasets_metric(predictions, references, jury_datasets, output_evaluate_datasets_metric):
    scores = jury_datasets(predictions=predictions, references=references)
    assert_almost_equal_dict(actual=scores, desired=output_evaluate_datasets_metric)


def test_evaluate_corpus(single_prediction_array, multiple_references, jury, output_evaluate_corpus):
    scores = jury(predictions=single_prediction_array, references=multiple_references)
    assert_almost_equal_dict(
        actual=scores, desired=output_evaluate_corpus, exclude_paths="root['bertscore']['hashcode']"
    )


def test_evaluate_multiple_predictions(
    multiple_predictions, multiple_references, jury, output_evaluate_multiple_predictions
):
    scores = jury(predictions=multiple_predictions, references=multiple_references)
    assert_almost_equal_dict(
        actual=scores, desired=output_evaluate_multiple_predictions, exclude_paths="root['bertscore']['hashcode']"
    )


def test_evaluate_sequence_classification(
    predictions_sequence_classification,
    references_sequence_classification,
    jury_sequence_classification,
    output_evaluate_sequence_classification,
):
    scores = jury_sequence_classification(
        predictions=predictions_sequence_classification, references=references_sequence_classification
    )
    assert_almost_equal_dict(actual=scores, desired=output_evaluate_sequence_classification)


def test_reduce_fn(predictions, references, jury):
    _reduce_fn = np.mean
    _non_reduce_fn = np.exp
    scores = jury(predictions=predictions, references=references, reduce_fn=_reduce_fn)

    assert all([scores[metric.resulting_name] is not None for metric in jury.metrics])

    with pytest.raises(ValueError):
        jury(predictions=predictions, references=references, reduce_fn=_non_reduce_fn)


def test_load_metric():
    from jury import load_metric
    from jury.metrics._core import Metric as JuryMetric

    assert isinstance(load_metric("squad"), JuryMetric)
    assert isinstance(load_metric("squad_v2"), datasets.Metric)

    with pytest.raises(FileNotFoundError):
        load_metric("abcdefgh")



import os
import re
import string
from typing import Any, Dict, List, Optional


class NestedSingleType:
    @staticmethod
    def is_iterable(obj):
        if isinstance(obj, str) or isinstance(obj, dict):
            return False
        try:
            iter(obj)
        except TypeError:
            return False
        return True

    @staticmethod
    def join(types: List[str]):
        nested_types = f"{types.pop(-1)}"

        for _type in types:
            nested_types = f"{_type}<{nested_types}>"
        return nested_types.lower()

    @classmethod
    def get_type(cls, obj, order: Optional[int] = None):
        _obj = obj

        types = []
        while cls.is_iterable(_obj):
            types.append(type(_obj).__name__)
            _obj = _obj[0]
        types.append(type(_obj).__name__)

        if order is not None:
            try:
                return types[order]
            except IndexError:
                return None

        return cls.join(types)


def remove_punctuations(text: str) -> str:
    pattern = r"[%s]" % re.escape(string.punctuation)
    text = re.sub(pattern, " ", text)
    return " ".join(text.split())


def bulk_remove_keys(obj: Dict, keys: List[str]) -> Dict:
    return {k: v for k, v in obj.items() if k not in keys}


def set_env(name: str, value: str):
    if not isinstance(value, str):
        raise ValueError(f"Expected type str for 'value', got {type(value)}.")
    os.environ[name] = value


def replace(a: List, obj: object, index=-1):
    del a[index]
    a.insert(index, obj)
    return a


def pop_item_from_dict(d: Dict[str, Any], key: str, default: Any = None, must_exists: bool = False):
    """
    Pops key from dict d if key exists, returns d otherwise.

    Args:
        d: (``Dict``) Dictionary for key to be removed.
        key: (``str``) Key name to remove from dictionary d.
        default: (``Any``) Default value to return if key not found.
        must_exists: (``bool``) Raises an exception if True when given key does not exists.

    Returns: Popped value for key if found, None otherwise.
    """
    if key not in d and must_exists:
        raise KeyError(f"'{key}' not found in '{d}'.")
    val = d.pop(key) if key in d else default
    return val



from jury.metrics._core import TaskMapper
from jury.metrics._core.utils import TaskNotAvailable
from jury.metrics.precision.precision_for_language_generation import PrecisionForLanguageGeneration
from jury.metrics.precision.precision_for_sequence_classification import PrecisionForSequenceClassification

__main_class__ = "Precision"


class Precision(TaskMapper):
    _TASKS = {
        "language-generation": PrecisionForLanguageGeneration,
        "sequence-classification": PrecisionForSequenceClassification,
    }



DEFAULT_METRICS = [
    {"metric_name": "bleu", "compute_kwargs": {"max_order": 1}},
    {"metric_name": "bleu", "compute_kwargs": {"max_order": 2}},
    {"metric_name": "bleu", "compute_kwargs": {"max_order": 3}},
    {"metric_name": "bleu", "compute_kwargs": {"max_order": 4}},
    {"metric_name": "meteor"},
    {"metric_name": "rouge"},
]



import numpy as np
import pytest

from jury.metrics._core.utils import is_reduce_fn
from jury.utils import bulk_remove_keys, remove_punctuations


@pytest.fixture
def raw_text():
    return '!*This\'s, a (test); text. Testing: 123 alpha123.  Multiple    spaces     "uneven sizes", and-also-this.'


@pytest.fixture
def preprocessed_text():
    return "This s a test text Testing 123 alpha123 Multiple spaces uneven sizes and also this"


def test_remove_punctuations(raw_text, preprocessed_text):
    assert remove_punctuations(raw_text) == preprocessed_text


def test_bulk_remove_keys():
    test_dict = {"a": 0, "b": 1, "c": 2}
    remove_keys = ["a", "b"]
    resulting_dict = {"c": 2}
    assert bulk_remove_keys(test_dict, remove_keys) == resulting_dict


def test_is_reduce_fn():
    assert not is_reduce_fn(np.exp)
    assert is_reduce_fn(np.mean)



import pytest

from jury import Jury
from jury.metrics import AutoMetric
from tests.jury.conftest import get_expected_output
from tests.utils import assert_almost_equal_dict


@pytest.fixture(scope="module")
def jury_language_generation():
    metric = AutoMetric.load("recall")
    return Jury(metrics=metric)


@pytest.fixture(scope="module")
def jury_sequence_classification():
    metric = AutoMetric.load("recall", task="sequence-classification")
    return Jury(metrics=metric)


@pytest.fixture
@get_expected_output(prefix="metrics")
def output_basic_language_generation():
    return output_basic_language_generation.output


@pytest.fixture
@get_expected_output(prefix="metrics")
def output_multiple_ref_language_generation():
    return output_multiple_ref_language_generation.output


@pytest.fixture
@get_expected_output(prefix="metrics")
def output_multiple_pred_multiple_ref_language_generation():
    return output_multiple_pred_multiple_ref_language_generation.output


@pytest.fixture
@get_expected_output(prefix="metrics")
def output_basic_sequence_classification():
    return output_basic_sequence_classification.output


@pytest.fixture
@get_expected_output(prefix="metrics")
def output_multiple_ref_sequence_classification():
    return output_multiple_ref_sequence_classification.output


@pytest.fixture
@get_expected_output(prefix="metrics")
def output_multiple_pred_multiple_ref_sequence_classification():
    return output_multiple_pred_multiple_ref_sequence_classification.output


def test_basic_language_generation(predictions, references, jury_language_generation, output_basic_language_generation):
    scores = jury_language_generation(predictions=predictions, references=references)
    assert_almost_equal_dict(actual=scores, desired=output_basic_language_generation)


def test_multiple_ref_language_generation(
    predictions, multiple_references, jury_language_generation, output_multiple_ref_language_generation
):
    scores = jury_language_generation(predictions=predictions, references=multiple_references)
    assert_almost_equal_dict(actual=scores, desired=output_multiple_ref_language_generation)


def test_multiple_pred_multiple_ref_language_generation(
    multiple_predictions,
    multiple_references,
    jury_language_generation,
    output_multiple_pred_multiple_ref_language_generation,
):
    scores = jury_language_generation(predictions=multiple_predictions, references=multiple_references)
    assert_almost_equal_dict(actual=scores, desired=output_multiple_pred_multiple_ref_language_generation)


def test_basic_sequence_classification(
    predictions_sequence_classification,
    references_sequence_classification,
    jury_sequence_classification,
    output_basic_sequence_classification,
):
    scores = jury_sequence_classification(
        predictions=predictions_sequence_classification, references=references_sequence_classification
    )
    assert_almost_equal_dict(actual=scores, desired=output_basic_sequence_classification)


def test_multiple_ref_sequence_classification(
    predictions_sequence_classification,
    multiple_references_sequence_classification,
    jury_sequence_classification,
    output_multiple_ref_sequence_classification,
):
    scores = jury_sequence_classification(
        predictions=predictions_sequence_classification, references=multiple_references_sequence_classification
    )
    assert_almost_equal_dict(actual=scores, desired=output_multiple_ref_sequence_classification)


def test_multiple_pred_multiple_ref_sequence_classification(
    multiple_predictions_sequence_classification,
    multiple_references_sequence_classification,
    jury_sequence_classification,
    output_multiple_pred_multiple_ref_sequence_classification,
):
    scores = jury_sequence_classification(
        predictions=multiple_predictions_sequence_classification, references=multiple_references_sequence_classification
    )
    assert_almost_equal_dict(actual=scores, desired=output_multiple_pred_multiple_ref_sequence_classification)



from jury.metrics._core import MetricAlias
from jury.metrics.seqeval.seqeval_for_sequence_classification import SeqevalForLanguageGeneration

__main_class__ = "Seqeval"


class Seqeval(MetricAlias):
    _SUBCLASS = SeqevalForLanguageGeneration



import os.path
from pathlib import Path

TESTS_JURY_DIR = Path(os.path.abspath(os.path.dirname(__file__)))
TESTS_DIR = TESTS_JURY_DIR.parent
TEST_DATA_DIR = TESTS_DIR / "test_data"
EXPECTED_OUTPUTS = TEST_DATA_DIR / "expected_outputs"



from jury.metrics._core import MetricAlias
from jury.metrics.squad.squad_for_language_generation import SquadForLanguageGeneration

__main_class__ = "Squad"


class Squad(MetricAlias):
    _SUBCLASS = SquadForLanguageGeneration



from pathlib import Path

import pytest

from tests.jury import TEST_DATA_DIR
from tests.jury.conftest import get_expected_output
from tests.utils import assert_almost_equal_dict, shell_capture


@pytest.fixture
@get_expected_output(prefix=None)
def output_cli():
    return output_cli.output


def test_cli(output_cli):
    predictions_file = Path(TEST_DATA_DIR) / "cli" / "predictions.txt"
    references_file = Path(TEST_DATA_DIR) / "cli" / "references.txt"

    cmd = f"jury eval --predictions {predictions_file} --references {references_file}"

    out = shell_capture(cmd)
    assert_almost_equal_dict(actual=out, desired=output_cli)



import io
import os
import platform
import re
from typing import List

import setuptools


def get_long_description():
    base_dir = os.path.abspath(os.path.dirname(__file__))
    with io.open(os.path.join(base_dir, "README.md"), encoding="utf-8") as f:
        return f.read()


def get_requirements():
    with open("requirements.txt") as f:
        return f.read().splitlines()


def get_version():
    current_dir = os.path.abspath(os.path.dirname(__file__))
    version_file = os.path.join(current_dir, "jury", "__init__.py")
    with io.open(version_file, encoding="utf-8") as f:
        return re.search(r'^__version__ = [\'"]([^\'"]*)[\'"]', f.read(), re.M).group(1)


def add_pywin(reqs: List[str]) -> None:
    if platform.system() == "Windows":
        # Latest PyWin32 build (301) fails, required for sacrebleu
        ext_package = ["pywin32==228"]
    else:
        ext_package = []
    reqs.extend(ext_package)


_DEV_REQUIREMENTS = [
    "black==21.7b0",
    "deepdiff==5.5.0",
    "flake8==3.9.2",
    "isort==5.9.2",
    "jiwer>=2.2.0",
    "pytest>=6.2.4",
    "pytest-cov>=2.12.1",
    "pytest-timeout>=1.4.2",
]

_METRIC_REQUIREMENTS = ["sacrebleu>=2.0.0", "bert_score==0.3.10", "seqeval==1.2.2"]
add_pywin(_METRIC_REQUIREMENTS)

extras = {
    "metrics": _METRIC_REQUIREMENTS,
    "dev": _DEV_REQUIREMENTS + _METRIC_REQUIREMENTS,
}


setuptools.setup(
    name="jury",
    version=get_version(),
    author="",
    license="MIT",
    description="Evaluation toolkit for neural language generation.",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/obss/jury",
    packages=setuptools.find_packages(exclude=["tests"]),
    python_requires=">=3.7",
    install_requires=get_requirements(),
    extras_require=extras,
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "jury=jury.cli:app",
        ],
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Education",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    keywords="machine-learning, deep-learning, ml, pytorch, NLP, evaluation, question-answering, question-generation",
)



# coding=utf-8
# Copyright 2020 Open Business Software Solutions, The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
""" SQuAD metric. The part of this file is adapted from SacreBLEU implementation
of datasets package. See
https://github.com/huggingface/datasets/blob/master/metrics/squad/squad.py"""
import os
from typing import Callable, Dict, List

import datasets
import numpy as np
import pandas as pd

from jury.collator import Collator
from jury.metrics._core import MetricForLanguageGeneration
from jury.metrics._core.utils import download
from jury.utils import NestedSingleType

_CITATION = """\
@inproceedings{Rajpurkar2016SQuAD10,
  title={SQuAD: 100, 000+ Questions for Machine Comprehension of Text},
  author={Pranav Rajpurkar and Jian Zhang and Konstantin Lopyrev and Percy Liang},
  booktitle={EMNLP},
  year={2016}
}
"""

_DESCRIPTION = """
This metric wrap the official scoring script for version 1 of the Stanford Question Answering Dataset (SQuAD).

Stanford Question Answering Dataset (SQuAD) is a reading comprehension dataset, consisting of questions posed by
crowdworkers on a set of Wikipedia articles, where the answer to every question is a segment of text, or span,
from the corresponding reading passage, or the question might be unanswerable.
"""

_KWARGS_DESCRIPTION = """
Computes SQuAD scores (F1 and EM).
Args:
    predictions: list of predictions to score. Each predictions
        should be a string with tokens separated by spaces. Optionally,
        List of question-answers dictionaries with the following key-values:
        - 'id': id of the question-answer pair as given in the references (see below)
        - 'prediction_text': the text of the answer
    references: list of reference for each prediction. Each
        reference should be a string with tokens separated by spaces. Optionally,
        List of question-answers dictionaries with the following key-values:
        - 'id': id of the question-answer pair (see above),
        - 'answers': a Dict in the SQuAD dataset format
            {
                'text': list of possible texts for the answer, as a list of strings
                'answer_start': list of start positions for the answer, as a list of ints
            }
            Note that answer_start values are not taken into account to compute the metric.
Returns:
    'exact_match': Exact match (the normalized answer exactly match the gold answer)
    'f1': The F-score of predicted tokens versus the gold answer
Examples:

    >>> squad = jury.load_metric("squad")
    >>> predictions = [["the cat is on the mat", "There is cat playing on the mat"], ["Look! a wonderful day."]]
    >>> references = [
        ["the cat is playing on the mat.", "The cat plays on the mat."], 
        ["Today is a wonderful day", "The weather outside is wonderful."]
    ]
    >>> results = squad.compute(predictions=predictions, references=references)
    >>> print(results)
    {'squad': {'exact_match': 0.0, 'f1': 74.02597402597402}}
"""


@datasets.utils.file_utils.add_start_docstrings(_DESCRIPTION, _KWARGS_DESCRIPTION)
class SquadForLanguageGeneration(MetricForLanguageGeneration):
    def _download_and_prepare(self, dl_manager) -> None:
        """
        Downloads and import the computation of squad score from the implementation
        of Squad computation from huggingface/datasets. See
        https://github.com/huggingface/datasets/blob/master/metrics/squad/evaluate.py
        """
        squad_source = "https://raw.githubusercontent.com/huggingface/datasets/master/metrics/squad/evaluate.py"
        squad_dest = os.path.join(self.data_dir, "squad_evaluate.py")
        download(
            source=squad_source,
            destination=squad_dest,
        )
        self.external_module_path = squad_dest

    def _info(self):
        return datasets.MetricInfo(
            description=_DESCRIPTION,
            citation=_CITATION,
            inputs_description=_KWARGS_DESCRIPTION,
            features=self._default_features,
            codebase_urls=["https://rajpurkar.github.io/SQuAD-explorer/"],
            reference_urls=["https://rajpurkar.github.io/SQuAD-explorer/"],
        )

    def _preprocess(self, predictions: Collator, references: Collator):
        if NestedSingleType.get_type(predictions, order=-1) == "str" and predictions.can_collapse():
            predictions = [{"prediction_text": pred, "id": str(i)} for i, pred in enumerate(predictions.collapse())]
        elif NestedSingleType.get_type(predictions, order=-1) == "str" and not predictions.can_collapse():
            _predictions = []
            for i, preds in enumerate(predictions):
                _predictions.append([{"prediction_text": pred, "id": str(i)} for pred in preds])
            predictions = _predictions
        if NestedSingleType.get_type(references, order=-1) == "str":
            references = [
                {"answers": {"answer_start": [-1], "text": Collator(ref).collapse()}, "id": str(i)}
                for i, ref in enumerate(references)
            ]
        return predictions, references

    def _compute_single_pred_single_ref(self, predictions: Collator, references: Collator, **kwargs):
        pred_dict = {prediction["id"]: prediction["prediction_text"] for prediction in predictions}
        dataset = [
            {
                "paragraphs": [
                    {
                        "qas": [
                            {
                                "answers": [{"text": answer_text} for answer_text in ref["answers"]["text"]],
                                "id": ref["id"],
                            }
                            for ref in references
                        ]
                    }
                ]
            }
        ]
        evaluation_fn = self._get_external_resource("squad_evaluate", attr="evaluate")
        score = evaluation_fn(dataset=dataset, predictions=pred_dict)
        for metric_, score_ in score.items():
            score[metric_] = score_ / 100
        return score

    def _compute_single_pred_multi_ref(
        self, predictions: Collator, references: Collator, reduce_fn: Callable = None, **kwargs
    ):
        return self._compute_single_pred_single_ref(predictions=predictions, references=references, reduce_fn=reduce_fn)

    def _compute_multi_pred_multi_ref(
        self, predictions: Collator, references: Collator, reduce_fn: Callable = None, **kwargs
    ):
        scores = []
        for preds, refs in zip(predictions, references):
            pred_scores = []
            for pred in preds:
                pred_scores.append(
                    self._compute_single_pred_multi_ref(
                        predictions=Collator([pred], keep=True),
                        references=Collator([refs], keep=True),
                        reduce_fn=reduce_fn,
                    )
                )
            reduced_score = self._reduce_multi_pred_scores(pred_scores, reduce_fn)
            scores.append(reduced_score)

        # Average reduced scores
        return self._reduce_multi_pred_scores(scores, np.mean)

    def _reduce_multi_pred_scores(self, results: List[Dict], reduce_fn) -> Dict:
        df = pd.DataFrame(results)
        return df.apply(reduce_fn, axis=0).to_dict()



from jury.metrics._core import TaskMapper
from jury.metrics.accuracy.accuracy_for_language_generation import AccuracyForLanguageGeneration
from jury.metrics.accuracy.accuracy_for_sequence_classification import AccuracyForSequenceClassification

__main_class__ = "Accuracy"


class Accuracy(TaskMapper):
    _TASKS = {
        "language-generation": AccuracyForLanguageGeneration,
        "sequence-classification": AccuracyForSequenceClassification,
    }
